==> selene-0.2.0.tar.gz: lib/src/ws/websocket.dart <==
part of selene;

/// The default implementation of [WSBase].
class _WebSocketImpl extends WSBase {
  final Logger log = new Logger('Selene-WebSocket');

  /// Creates a new [_WebSocketImpl].
  _WebSocketImpl(this.client) {
    _payloadHandlers = {
      0: (_WSFrame frame) async {
        // OP 0 Dispatch
        _lastSequence = frame.sequenceNumber ?? _lastSequence;
        if (frame.eventName == 'READY') {
          sessionId = frame.data['session_id'];
        }
        await client.dispatcher._handle(frame.eventName, frame.data);
      },
      1: (_WSFrame frame) async {},
      7: (_WSFrame frame) async {},
      8: (_WSFrame frame) async {},
      9: (_WSFrame frame) async {
        // OP 9 Invalid Session
        print('Invalid Session');
      },
      10: (_WSFrame frame) async {
        // OP 10 Hello
        heartbeatInterval = frame.data['heartbeat_interval'];
        heartbeater = new Timer.periodic(
            new Duration(milliseconds: heartbeatInterval), (timer) {
          heartbeat();
        });
        // User should probably be executing `_WSBase.login` now
      },
      11: (_WSFrame frame) async {},
    };
  }

  /// The current heartbeat timer.
  Timer heartbeater;

  /// The current heartbeat interval.
  int heartbeatInterval;

  /// The current JSON encoder.
  JsonEncoder _encoder = new JsonEncoder();

  /// The current JSON decoder.
  JsonDecoder _decoder = new JsonDecoder();

  /// The current [Client].
  Client client;

  /// The current WebSocket event subscription.
  StreamSubscription<dynamic> _wsSubscription;

  /// The internal [WebSocket].
  transport.WebSocket _ws;

  /// A cached URI for the Discord gateway.
  Uri gatewayUri;

  /// Whether the gateway URI has been cached.
  bool get gatewayUriCached =>
      (gatewayUri != null) && gatewayUri.toString().isNotEmpty;

  /// Whether the gateway URI has not been cached.
  bool get gatewayUriNotCached => !gatewayUriCached;

  /// The last sequence number this client received.
  int _lastSequence;

  /// (For resuming) The Session ID of this session.
  String sessionId;

  /// The recommended shard count.
  ///
  /// Will be null unless [requestGatewayUri] with `shards = true` has been executed.
  int recommendedShardCount;

  /// Whether this WebSocket is using sharding.
  bool get isSharding => shardId != null && shardTotal != null;

  /// Whether this WebSocket is not using sharding.
  bool get isNotSharding => !isSharding;

  /// (If sharding) This shard ID.
  int shardId;

  /// (If sharding) The total amount of shards in this client.
  int shardTotal;

  /// A list of all payload handlers.
  Map<int, PayloadHandler> _payloadHandlers;

  /// The token to authenticate with.
  String token;

  /// The type of the supplied token.
  String tokenType;

  /// Whether this client is attempting to resume.
  bool isResuming = false;

  /// Whether this client is not attempting to resume.
  bool get isNotResuming => !isResuming;

  /// Requests a new gateway URI (and sets [recommendedShardCount] if `shards = true`) from Discord.
  Future<Uri> requestGatewayUri(bool shards) async {
    if (shards) {
      var shardResponse = await client._rest.getGatewayWithShards();
      var shardBody = shardResponse.body.asJson();
      recommendedShardCount = shardBody['shards'];
      return Uri.parse(shardBody['url']);
    } else {
      var response = await client._rest.getGateway();
      var body = response.body.asJson();
      recommendedShardCount = null;
      return Uri.parse(body['url']);
    }
  }

  /// Sends an identify payload over the WebSocket.
  void sendIdentify() {
    var frame =
        new _IdentifyFrame(token, Platform.operatingSystem, 'Selene', 'Selene');
    if (isSharding) {
      frame.data['shard'] = [shardId, shardTotal];
    }
    sendFrame(frame);
  }

  /// Sends a resume payload over the WebSocket.
  void sendResume() {
    var frame = new _WSFrame();
    frame.data = <String, dynamic>{
      'token': token,
      'session_id': sessionId,
      'seq': _lastSequence
    };
    frame.opCode = 6; // OP 6 - Resume
    sendFrame(frame);
  }

  /// Sends a heartbeat payload over the WebSocket.
  void heartbeat() {
    sendFrame(new _HeartbeatFrame(_lastSequence));
  }

  /// Sends a [_WSFrame] over the WebSocket.
  void sendFrame(_WSFrame frame) {
    if (frame.direction == FrameDirection.Inbound) {
      throw new StateError(
          'Cannot send an inbound WebSocket frame');
    }
    _ws.add(_encoder.convert(frame.build()));
  }

  @override
  Future start([bool reconnecting = false]) async {
    if (gatewayUriNotCached) {
      var reqUri = await requestGatewayUri(isSharding);
      gatewayUri = Uri.parse(reqUri.toString() + "?v=6&encoding=json");
    }
    isResuming = reconnecting;
    _ws = await transport.WebSocket
        .connect(gatewayUri, transportPlatform: vmTransportPlatform);

    _wsSubscription = _ws.listen(handleWSMessage,
        onError: handleWSError, onDone: handleWSClose);
  }

  @override
  Future login(String token, {String type = 'Bot'}) async {
    this.token = token;
    this.tokenType = type;
    if (isNotResuming)
      sendIdentify();
    else
      sendResume();
  }

  /// Handles a message received from the WebSocket.
  Future handleWSMessage(dynamic message) async {
    var frame = new _WSFrame()..update(_decoder.convert(message));
    frame.direction = FrameDirection.Inbound;
    var handler = _payloadHandlers[frame.opCode];
    if (handler == null) {
      print(
          '[Selene WebSocket] Warning: Received an OP code that doesn\'t have a handler, library version (Selene v${versionNumber}) be out-of-date.');
      return;
    }
    await handler(frame);
  }

  /// Handles an error received from the WebSocket.
  Future handleWSError(dynamic error) async {}

  /// Handles the WebSocket in the event that it is closed.
  Future handleWSClose() async {}

  @override
  Future stop() async {
    await _wsSubscription.cancel();
    await _ws.close(0, 'Client disconnecting');
  }
}

==> selene-0.3.1.tar.gz: lib/src/ws/websocket.dart <==
part of selene;

/// The default implementation of [WSBase].
class _WebSocketImpl extends WSBase {
  final Logger log = new Logger('Selene-WebSocket');

  /// Creates a new [_WebSocketImpl].
  _WebSocketImpl(this.client) {
    _payloadHandlers = {
      0: (_WSFrame frame) async {
        // OP 0 Dispatch
        _lastSequence = frame.sequenceNumber ?? _lastSequence;
        if (frame.eventName == 'READY') {
          sessionId = frame.data['session_id'];
        }
        await client.dispatcher._handle(frame.eventName, frame.data);
      },
      1: (_WSFrame frame) async {},
      7: (_WSFrame frame) async {},
      8: (_WSFrame frame) async {},
      9: (_WSFrame frame) async {
        // OP 9 Invalid Session
        print('Invalid Session');
      },
      10: (_WSFrame frame) async {
        // OP 10 Hello
        heartbeatInterval = frame.data['heartbeat_interval'];
        heartbeater = new Timer.periodic(
            new Duration(milliseconds: heartbeatInterval), (timer) {
          heartbeat();
        });
        if (isNotResuming)
          sendIdentify();
        else
          sendResume();
      },
      11: (_WSFrame frame) async {},
    };
  }

  /// The current heartbeat timer.
  Timer heartbeater;

  /// The current heartbeat interval.
  int heartbeatInterval;

  /// The current [Client].
  Client client;

  /// The current WebSocket event subscription.
  StreamSubscription<dynamic> _wsSubscription;

  /// The internal [WebSocket].
  transport.WebSocket _ws;

  /// A cached URI for the Discord gateway.
  Uri gatewayUri;

  /// Whether the gateway URI has been cached.
  bool get gatewayUriCached =>
      (gatewayUri != null) && gatewayUri.toString().isNotEmpty;

  /// Whether the gateway URI has not been cached.
  bool get gatewayUriNotCached => !gatewayUriCached;

  /// The last sequence number this client received.
  int _lastSequence;

  /// (For resuming) The Session ID of this session.
  String sessionId;

  /// The recommended shard count.
  ///
  /// Will be null unless [requestGatewayUri] with `shards = true` has been executed.
  int recommendedShardCount;

  /// Whether this WebSocket is using sharding.
  bool get isSharding => shardId != null && shardTotal != null;

  /// Whether this WebSocket is not using sharding.
  bool get isNotSharding => !isSharding;

  /// (If sharding) This shard ID.
  int shardId;

  /// (If sharding) The total amount of shards in this client.
  int shardTotal;

  /// A list of all payload handlers.
  Map<int, PayloadHandler> _payloadHandlers;

  /// The token to authenticate with.
  String token;

  /// The type of the supplied token.
  String tokenType;

  /// Whether this client is attempting to resume.
  bool isResuming = false;

  /// Whether this client is not attempting to resume.
  bool get isNotResuming => !isResuming;

  /// Whether this client is receiving played back events after a resume.
  bool isPlaybackMode = false;

  /// Whether this client is not receiving played back events after a resume.
  bool get isNotPlaybackMode => !isPlaybackMode;

  /// Requests a new gateway URI (and sets [recommendedShardCount] if `shards = true`) from Discord.
  Future<Uri> requestGatewayUri(bool shards) async {
    if (shards) {
      var shardResponse =
      await client._rest.makeNewRequest('/gateway/bot', 'GET');
      var shardBody = shardResponse.body.asJson();
      recommendedShardCount = shardBody['shards'];
      return Uri.parse(shardBody['url']);
    } else {
      var response = await client._rest.makeNewRequest(
          '/gateway', 'GET', null, RequestOptions._default..authorize = false);
      var body = response.body.asJson();
      recommendedShardCount = null;
      return Uri.parse(body['url']);
    }
  }

  /// Sends an identify payload over the WebSocket.
  void sendIdentify() {
    var frame =
        new _IdentifyFrame(token, Platform.operatingSystem, 'Selene', 'Selene');
    if (isSharding) {
      frame.data['shard'] = [shardId, shardTotal];
    }
    sendFrame(frame);
  }

  /// Sends a resume payload over the WebSocket.
  void sendResume() {
    var frame = new _WSFrame();
    frame.data = <String, dynamic>{
      'token': token,
      'session_id': sessionId,
      'seq': _lastSequence
    };
    frame.opCode = 6; // OP 6 - Resume
    sendFrame(frame);
  }

  /// Sends a heartbeat payload over the WebSocket.
  void heartbeat() {
    sendFrame(new _HeartbeatFrame(_lastSequence));
  }

  /// Sends a [_WSFrame] over the WebSocket.
  void sendFrame(_WSFrame frame) {
    if (frame.direction == FrameDirection.Inbound) {
      throw new StateError('Cannot send an inbound WebSocket frame');
    }
    _ws.add(client._json.serialize(frame.build()));
  }

  @override
  Future start([bool reconnecting = false]) async {
    if (gatewayUriNotCached) {
      var reqUri = await requestGatewayUri(isSharding);
      gatewayUri = Uri.parse(reqUri.toString() + "?v=6&encoding=json");
    }
    isResuming = reconnecting;
    _ws = await transport.WebSocket
        .connect(gatewayUri, transportPlatform: vmTransportPlatform);

    _wsSubscription = _ws.listen(handleWSMessage,
        onError: handleWSError, onDone: handleWSClose);
  }

  @override
  void login(String token, {String type = 'Bot'}) {
    this.token = token;
    this.tokenType = type;
  }

  /// Handles a message received from the WebSocket.
  Future handleWSMessage(dynamic message) async {
    var frame = new _WSFrame()
      ..update(client._json.deserialize(message));
    frame.direction = FrameDirection.Inbound;
    var handler = _payloadHandlers[frame.opCode];
    if (handler == null) {
      print(
          '[Selene WebSocket] Warning: Received an OP code that doesn\'t have a handler, library version (Selene v${versionNumber}) be out-of-date.');
      return;
    }
    await handler(frame);
  }

  /// Handles an error received from the WebSocket.
  Future handleWSError(dynamic error) async {}

  /// Handles the WebSocket in the event that it is closed.
  Future handleWSClose() async {}

  @override
  Future stop() async {
    await _wsSubscription.cancel();
    await _ws.close(0, 'Client disconnecting');
  }
}
